package id.co.hanoman.ngdemo;

import java.util.Arrays;

import org.joda.time.LocalDate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.orm.jpa.EntityScan;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.context.request.async.DeferredResult;

import com.fasterxml.classmate.TypeResolver;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.senomas.boot.security.LoginUser;
import com.senomas.common.RateLimit;

import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.builders.ResponseMessageBuilder;
import springfox.documentation.schema.AlternateTypeRules;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.schema.WildcardType;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

@SpringBootApplication
@EnableSwagger2
@ComponentScan({ "id.co.hanoman.ngdemo", "com.senomas.boot", "com.senomas.common.loggerfilter",
		"com.senomas.common.logback" })
@EntityScan({ "id.co.hanoman.ngdemo" })
@EnableJpaRepositories({ "id.co.hanoman.ngdemo", "com.senomas.boot" })
@EnableMongoRepositories({ "com.senomas.common.logback" })
@EnableAsync
public class Application {

	@Autowired
	TypeResolver typeResolver;

	@Autowired
	JedisConnectionFactory jedisConnFactory;

	@Bean
	public Docket getApi() {
		// @formatter:off
		return new Docket(DocumentationType.SWAGGER_2).select().apis(RequestHandlerSelectors.any()).paths(paths())
				.build().pathMapping("/").directModelSubstitute(LocalDate.class, String.class)
				.genericModelSubstitutes(ResponseEntity.class)
				.alternateTypeRules(AlternateTypeRules.newRule(
						typeResolver.resolve(DeferredResult.class,
								typeResolver.resolve(ResponseEntity.class, WildcardType.class)),
						typeResolver.resolve(WildcardType.class)))
				.useDefaultResponseMessages(false)
				.globalResponseMessage(RequestMethod.GET, Arrays.asList(new ResponseMessageBuilder().code(500)
						.message("500 message").responseModel(new ModelRef("Error")).build()));
		// @formatter:on
	}

	@Bean
	public RedisTemplate<String, LoginUser> loginUserRedisTemplate() {
		RedisTemplate<String, LoginUser> users = new RedisTemplate<>();
		users.setConnectionFactory(jedisConnFactory);
		users.setKeySerializer(new StringRedisSerializer());
		users.setValueSerializer(new Jackson2JsonRedisSerializer<>(LoginUser.class));
		return users;
	}

	@Bean
	public RedisTemplate<String, RateLimit> rateLimitRedisTemplate() {
		RedisTemplate<String, RateLimit> rates = new RedisTemplate<>();
		rates.setConnectionFactory(jedisConnFactory);
		rates.setKeySerializer(new StringRedisSerializer());
		rates.setValueSerializer(new Jackson2JsonRedisSerializer<>(RateLimit.class));
		return rates;
	}

	private Predicate<String> paths() {
		return Predicates.or(PathSelectors.regex("/s/.*"), PathSelectors.regex("/api/.*"));
	}

	public static void main(String[] args) {
		SpringApplication.run(Application.class, args);
	}

}
