package id.co.hanoman.ngdemo;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import com.senomas.boot.security.LoginUser;
import com.senomas.boot.security.SecurityConfiguration;
import com.senomas.boot.security.domain.SecurityUser;
import com.senomas.boot.security.service.TokenService;
import com.senomas.boot.security.service.UserService;

@Component
@EnableConfigurationProperties(SecurityConfiguration.class)
public class TokenServiceImpl implements TokenService {
	private static final Logger log = LoggerFactory.getLogger(TokenServiceImpl.class);

	@Autowired
	SecurityConfiguration config;

	@Autowired
	UserService userService;

	@Autowired
	RedisTemplate<String, LoginUser> users;

	@Autowired
	StringRedisTemplate tokens;

	public TokenServiceImpl() {
		log.debug("init");
	}

	@Override
	public LoginUser get(String token) {
		return users.opsForValue().get(token);
	}

	@Override
	public LoginUser create(SecurityUser user) {
		String token = UUID.randomUUID().toString();
		for (int i = 0; i < 1000 && users.hasKey(token); i++) {
			try {
				Thread.sleep(10);
			} catch (Exception e) {
			}
			token = UUID.randomUUID().toString();
		}
		if (users.hasKey(token))
			throw new RuntimeException("Too many tokens");
		LoginUser loginUser = new LoginUser(token, user, userService.getRoles(user));
		String oldToken = tokens.opsForValue().get(loginUser.getUser().getLogin());
		if (oldToken != null) {
			if (log.isTraceEnabled())
				log.trace("Remove old token " + oldToken + " new " + token);
			users.delete(oldToken);
		}
		users.opsForValue().set(token, loginUser, config.getTokenExpiry(), TimeUnit.SECONDS);
		tokens.opsForValue().set(loginUser.getUser().getLogin(), token);
		return loginUser;
	}

	@Override
	public boolean hasSalt(String salt) {
		return true;
	}

	@Override
	public String getSalt(String salt) {
		return null;
	}

	@Override
	public void setSalt(String salt, String value) {
	}

}
