package id.co.hanoman.ngdemo.domain;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;

import com.senomas.boot.security.domain.SecurityUser;

@Entity
public class AppUser implements SecurityUser {

	@Id
	@Column(length = 64)
	private String login;

	@Column(nullable = false, unique = true, length = 256)
	private String fullName;

	@Column(nullable = false, length = 256)
	private String passcode;

	@Column(nullable = true, length = 256)
	private String loginToken;

	@Column(nullable = true)
	private Date lastLogin;

	@ManyToMany(cascade = CascadeType.ALL)
	@JoinTable(name = "app_user_role", joinColumns = @JoinColumn(name = "user", referencedColumnName = "login"), inverseJoinColumns = @JoinColumn(name = "role", referencedColumnName = "code"))
	private List<AppRole> roles = new LinkedList<>();

	public String getLogin() {
		return login;
	}

	public void setLogin(String login) {
		this.login = login;
	}

	public String getFullName() {
		return fullName;
	}

	public void setFullName(String fullName) {
		this.fullName = fullName;
	}

	public String getPasscode() {
		return passcode;
	}

	public void setPasscode(String passcode) {
		this.passcode = passcode;
	}

	public String getLoginToken() {
		return loginToken;
	}

	public void setLoginToken(String loginToken) {
		this.loginToken = loginToken;
		this.lastLogin = new Date();
	}

	public Date getLastLogin() {
		return lastLogin;
	}

	public Collection<AppRole> getRoles() {
		return roles;
	}

	public void setRoles(Collection<AppRole> roles) {
		this.roles.clear();
		this.roles.addAll(roles);
		sortRole();
	}

	public void addRole(AppRole role) {
		this.roles.add(role);
		sortRole();
	}

	public void removeRole(AppRole role) {
		for (Iterator<AppRole> ir = roles.iterator(); ir.hasNext();) {
			AppRole r = ir.next();
			if (r.getCode().equals(role.getCode())) {
				ir.remove();
			}
		}
	}

	protected void sortRole() {
		Collections.sort(this.roles, new Comparator<AppRole>() {
			@Override
			public int compare(AppRole r1, AppRole r2) {
				return r1.getCode().compareTo(r2.getCode());
			}
		});
	}
}
